@@ -1,10 +1,7 @@
 <?php
 /**
- * Base class for managing database schemes and handling database-specific SQL
- * dialects and quoting.
- *
  * Copyright 2007 Maintainable Software, LLC
- * Copyright 2008-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2008-2014 Horde LLC (http://www.horde.org/)
  *
  * @author     Mike Naberezny <mike@maintainable.com>
  * @author     Derek DeVries <derek@maintainable.com>
@@ -17,6 +14,9 @@
  */
 
 /**
+ * Base class for managing database schemes and handling database-specific SQL
+ * dialects and quoting.
+ *
  * @author     Mike Naberezny <mike@maintainable.com>
  * @author     Derek DeVries <derek@maintainable.com>
  * @author     Chuck Hagenbuch <chuck@horde.org>
@@ -259,7 +259,10 @@ abstract class Horde_Db_Adapter_Base_Schema
      *
      * @return string  The quoted column name.
      */
-    abstract public function quoteColumnName($name);
+    public function quoteColumnName($name)
+    {
+        return '"' . str_replace('"', '""', $name) . '"';
+    }
 
     /**
      * Returns a quoted form of the table name.
@@ -282,7 +285,7 @@ abstract class Horde_Db_Adapter_Base_Schema
      */
     public function quoteTrue()
     {
-        return "'t'";
+        return '1';
     }
 
     /**
@@ -292,7 +295,7 @@ abstract class Horde_Db_Adapter_Base_Schema
      */
     public function quoteFalse()
     {
-        return "'f'";
+        return '0';
     }
 
     /**
@@ -334,10 +337,7 @@ abstract class Horde_Db_Adapter_Base_Schema
      *
      * @return array  A database type map.
      */
-    public function nativeDatabaseTypes()
-    {
-        return array();
-    }
+    abstract public function nativeDatabaseTypes();
 
     /**
      * Returns the maximum length a table alias can have.
@@ -417,6 +417,27 @@ abstract class Horde_Db_Adapter_Base_Schema
      */
     abstract public function columns($tableName, $name = null);
 
+    /**
+     * Returns a table column.
+     *
+     * @since Horde_Db 2.1.0
+     *
+     * @param string $tableName   A table name.
+     * @param string $columnName  A column name.
+     *
+     * @throws Horde_Db_Exception if column not found.
+     * @return Horde_Db_Adapter_Base_Column  A column object.
+     */
+    public function column($tableName, $columnName)
+    {
+        foreach ($this->columns($tableName) as $column) {
+            if ($column->getName() == $columnName) {
+                return $column;
+            }
+        }
+        throw new Horde_Db_Exception("$tableName does not have a column '$columnName'");
+    }
+
     /**
      * Creates a new table.
      *
@@ -545,7 +566,7 @@ abstract class Horde_Db_Adapter_Base_Schema
                         $tableDefinition->toSql(),
                         $opts);
 
-        return $this->execute($sql);
+        $this->execute($sql);
     }
 
     /**
@@ -729,6 +750,8 @@ abstract class Horde_Db_Adapter_Base_Schema
      *                                  - name: (string) the index name.
      *                                  - unique: (boolean) create a unique
      *                                            index?
+     *
+     * @return string  The index name. @since Horde_Db 2.1.0
      */
     public function addIndex($tableName, $columnName, $options = array())
     {
@@ -737,7 +760,7 @@ abstract class Horde_Db_Adapter_Base_Schema
         $columnNames = (array)$columnName;
         $indexName = empty($options['name'])
             ? $this->indexName($tableName, array('column' => $columnNames))
-            : $options['name'];
+            : $this->indexName($tableName, $options);
         foreach ($columnNames as &$colName) {
             $colName = $this->quoteColumnName($colName);
         }
@@ -747,8 +770,9 @@ abstract class Horde_Db_Adapter_Base_Schema
                        $this->quoteColumnName($indexName),
                        $this->quoteTableName($tableName),
                        implode(', ', $columnNames));
+        $this->execute($sql);
 
-        return $this->execute($sql);
+        return $indexName;
     }
 
     /**
@@ -875,7 +899,9 @@ abstract class Horde_Db_Adapter_Base_Schema
         }
 
         $sql = is_array($native) ? $native['name'] : $native;
-        if ($type == 'decimal') {
+        if ($type == 'decimal' ||
+            is_array($native) && (isset($native['precision']) || isset($native['scale'])) ||
+            isset($precision) || isset($scale)) {
             $nativePrec  = isset($native['precision']) ? $native['precision'] : null;
             $nativeScale = isset($native['scale'])     ? $native['scale']     : null;
 
@@ -906,6 +932,8 @@ abstract class Horde_Db_Adapter_Base_Schema
      *
      * @param string $sql     Existing SQL definition for a column.
      * @param array $options  Column options:
+     *                        - column: (Horde_Db_Adapter_Base_ColumnDefinition
+     *                          The column definition class.
      *                        - null: (boolean) Whether to allow NULL values.
      *                        - default: (mixed) Default column value.
      *                        - autoincrement: (boolean) Whether the column is
@@ -1113,7 +1141,7 @@ abstract class Horde_Db_Adapter_Base_Schema
      */
     protected function _clearTableCache($tableName)
     {
-        $this->_cache->set('tables/columns/' . $tableName, '');
-        $this->_cache->set('tables/indexes/' . $tableName, '');
+        $this->cacheWrite('tables/columns/' . $tableName, '');
+        $this->cacheWrite('tables/indexes/' . $tableName, '');
     }
 }
